/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.internal;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.ActionFactory;
import org.fortiss.tooling.kernel.base.EObjectAction;
import org.fortiss.tooling.kernel.model.IRemovable;
import org.fortiss.tooling.kernel.services.IActionService;
import org.fortiss.tooling.kernel.services.ICommandStackService;
import org.fortiss.tooling.kernel.services.IPersistencyService;
import org.fortiss.tooling.kernel.util.EObjectSelectionUtils;

/**
 * This class implements the {@link IActionService} interface.
 * 
 * @author hoelzlf
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED Hash: 144942DA212910AA70C65FE92CCFFA75
 */
public class ActionService implements IActionService {

	/** The group id of the global default action group. */
	public static final String GROUP_GLOBAL_DEFAULTS = "globalDefaults";

	/** The global undo action. */
	public final EObjectAction globalUndoAction = new EObjectAction("Undo",
			PlatformUI.getWorkbench().getSharedImages()
					.getImageDescriptor(ISharedImages.IMG_TOOL_UNDO)) {

		@Override
		public void run() {
			if (getTarget() != null) {
				ICommandStackService.INSTANCE.undo(getTarget());
			}
		}
	};

	/** The global redo action. */
	public final EObjectAction globalRedoAction = new EObjectAction("Redo",
			PlatformUI.getWorkbench().getSharedImages()
					.getImageDescriptor(ISharedImages.IMG_TOOL_UNDO)) {

		@Override
		public void run() {
			if (getTarget() != null) {
				ICommandStackService.INSTANCE.redo(getTarget());
			}
		}
	};

	/** The global delete action. */
	public final EObjectAction globalDeleteAction = new EObjectAction("Delete",
			PlatformUI.getWorkbench().getSharedImages()
					.getImageDescriptor(ISharedImages.IMG_TOOL_DELETE)) {
		@Override
		public void run() {
			final EObject selectedObject = getTarget();
			if (selectedObject == null) {
				return;
			}
			ICommandStackService.INSTANCE.runAsCommand(selectedObject,
					new Runnable() {
						@Override
						public void run() {
							if (selectedObject instanceof IRemovable) {
								((IRemovable) selectedObject).remove();
							} else {
								EcoreUtil.delete(selectedObject, true);
							}
						}
					});
		}
	};

	/** {@inheritDoc} */
	@Override
	public void addGlobalDefaultActionSectionToMenu(IMenuManager menuManager) {
		menuManager.appendToGroup(GROUP_GLOBAL_DEFAULTS, globalDeleteAction);
		menuManager.appendToGroup(GROUP_GLOBAL_DEFAULTS, globalUndoAction);
		menuManager.appendToGroup(GROUP_GLOBAL_DEFAULTS, globalRedoAction);
	}

	/** {@inheritDoc} */
	@Override
	public void registerGlobalActions(IViewSite site) {
		fillActionBars(site.getActionBars());
	}

	/** {@inheritDoc} */
	@Override
	public void registerGlobalActions(IEditorSite site) {
		fillActionBars(site.getActionBars());
	}

	/** Fills the given action bar with the global actions. */
	private void fillActionBars(IActionBars actionBars) {
		globalUndoAction.setId(ActionFactory.UNDO.getId());
		globalUndoAction.setActionDefinitionId(ActionFactory.UNDO
				.getCommandId());
		actionBars.setGlobalActionHandler(ActionFactory.UNDO.getId(),
				globalUndoAction);

		globalRedoAction.setId(ActionFactory.REDO.getId());
		globalRedoAction.setActionDefinitionId(ActionFactory.REDO
				.getCommandId());
		actionBars.setGlobalActionHandler(ActionFactory.REDO.getId(),
				globalRedoAction);

		actionBars.updateActionBars();
	}

	/** {@inheritDoc} */
	@Override
	public void refresh() {
		EObject target = EObjectSelectionUtils
				.getCurrentSelectionFirstElement();

		if (target == null) {
			globalUndoAction.setEnabled(false);
			globalRedoAction.setEnabled(false);
			globalDeleteAction.setEnabled(false);
			return;
		}

		globalUndoAction.setTarget(target);
		globalUndoAction.setEnabled(ICommandStackService.INSTANCE
				.canUndo(target));

		globalRedoAction.setTarget(target);
		globalRedoAction.setEnabled(ICommandStackService.INSTANCE
				.canRedo(target));

		boolean canDelete = (target instanceof IRemovable)
				&& ((IRemovable) target).canRemove();
		canDelete = canDelete
				|| !IPersistencyService.INSTANCE.isTopLevelElement(target);
		globalDeleteAction.setTarget(target);
		globalDeleteAction.setEnabled(canDelete);
	}
}
