/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.internal;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.conqat.lib.commons.collections.CollectionUtils;
import org.conqat.lib.commons.collections.UnmodifiableList;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.kernel.interfaces.IStorageProvider;
import org.fortiss.tooling.kernel.interfaces.ITopLevelElementChangeListener;
import org.fortiss.tooling.kernel.interfaces.ITopLevelElementContext;
import org.fortiss.tooling.kernel.internal.storage.eclipse.EclipseResourceStorageProvider;
import org.fortiss.tooling.kernel.services.IPersistencyService;

/**
 * This class implements the {@link IPersistencyService} interface.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED Hash: 91A0548CFEB392918C83B0159A4638EC
 */
public class PersistencyService implements IPersistencyService {

	/** Stores the top-level element contexts. */
	private final List<ITopLevelElementContext> contextCache = new LinkedList<ITopLevelElementContext>();

	/** Stores the storage providers and their element caches. */
	private final Map<ITopLevelElementContext, IStorageProvider> storageProviderCache = new HashMap<ITopLevelElementContext, IStorageProvider>();

	/** Stores the top-level element listeners. */
	private final List<ITopLevelElementChangeListener> listeners = new ArrayList<ITopLevelElementChangeListener>();

	/** Constructor. */
	public PersistencyService() {
		// TODO (FH): replace with extension mechanism
		IStorageProvider provider = new EclipseResourceStorageProvider();
		for (ITopLevelElementContext context : provider
				.getTopLevelElementContexts()) {
			contextCache.add(context);
			storageProviderCache.put(context, provider);
		}
	}

	/** {@inheritDoc} */
	@Override
	public UnmodifiableList<ITopLevelElementContext> getTopLevelElementContexts() {
		return CollectionUtils.asUnmodifiable(contextCache);
	}

	/** {@inheritDoc} */
	@Override
	public boolean isTopLevelElement(EObject element) {
		for (ITopLevelElementContext context : contextCache) {
			if (context.getTopLevelElement() == element) {
				return true;
			}
		}
		return false;
	}

	/** {@inheritDoc} */
	@Override
	public boolean isDirty() {
		for (ITopLevelElementContext context : contextCache) {
			if (context.isDirty()) {
				return true;
			}
		}
		return false;
	}

	/** {@inheritDoc} */
	@Override
	public void doSave(IProgressMonitor monitor) {
		// TODO Auto-generated method stub

	}

	/** {@inheritDoc} */
	@Override
	public void refreshTopLevelElements(IStorageProvider provider) {
		if (provider == null) {
			return;
		}

		List<ITopLevelElementContext> providedElements = provider
				.getTopLevelElementContexts();
		List<ITopLevelElementContext> removedCacheElements = new LinkedList<ITopLevelElementContext>();
		for (ITopLevelElementContext top : contextCache) {
			if (provider == storageProviderCache.get(top)
					&& !providedElements.contains(top)) {
				removedCacheElements.add(top);
			}
		}
		for (ITopLevelElementContext top : providedElements) {
			if (!contextCache.contains(top)) {
				contextCache.add(top);
				storageProviderCache.put(top, provider);
				notifyListenersAboutAdd(top);
			}
		}
		for (ITopLevelElementContext top : removedCacheElements) {
			contextCache.remove(top);
			storageProviderCache.remove(top);
			notifyListenersAboutRemove(top);
		}
	}

	/** {@inheritDoc} */
	@Override
	public void addTopLevelElementListener(
			ITopLevelElementChangeListener listener) {
		if (!listeners.contains(listener)) {
			listeners.add(listener);
		}
	}

	/** {@inheritDoc} */
	@Override
	public void removeTopLevelElementListener(
			ITopLevelElementChangeListener listener) {
		listeners.remove(listener);
	}

	/** Notifies listener about top-level element adding. */
	private void notifyListenersAboutAdd(ITopLevelElementContext top) {
		for (ITopLevelElementChangeListener listener : listeners) {
			listener.topLevelElementAdded(top);
		}
	}

	/** Notifies listener about top-level element removal. */
	private void notifyListenersAboutRemove(ITopLevelElementContext top) {
		for (ITopLevelElementChangeListener listener : listeners) {
			listener.topLevelElementRemoved(top);
		}
	}

	/** {@inheritDoc} */
	@Override
	public ITopLevelElementContext getTopLevelElementContextFor(
			EObject modelElement) {
		while (modelElement != null) {
			for (ITopLevelElementContext context : contextCache) {
				if (context.getTopLevelElement() == modelElement) {
					return context;
				}
			}
			modelElement = modelElement.eContainer();
		}
		return null;
	}
}
