/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.internal.navigator;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.viewers.DecoratingLabelProvider;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IDecoratorManager;
import org.eclipse.ui.ISaveablePart;
import org.eclipse.ui.ISaveablesSource;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchPartConstants;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.Saveable;
import org.eclipse.ui.part.ViewPart;
import org.eclipse.ui.progress.UIJob;
import org.eclipse.ui.views.properties.IPropertySheetPage;
import org.eclipse.ui.views.properties.tabbed.ITabbedPropertySheetPageContributor;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;
import org.fortiss.tooling.kernel.interfaces.ICustomMenuContributor;
import org.fortiss.tooling.kernel.internal.NavigatorService;
import org.fortiss.tooling.kernel.services.IActionService;
import org.fortiss.tooling.kernel.services.IContextMenuService;
import org.fortiss.tooling.kernel.services.IEditorService;
import org.fortiss.tooling.kernel.services.INavigatorService;
import org.fortiss.tooling.kernel.services.IPersistencyService;
import org.fortiss.tooling.kernel.services.IPropertiesService;
import org.fortiss.tooling.kernel.util.EObjectSelectionUtils;

/**
 * {@link ViewPart} of the model navigator provided by the tooling kernel.
 * 
 * @author hoelzlf
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED Hash: 03CFC0F958152C349546C3F63E5CBF6F
 */
public final class NavigatorViewPart extends ViewPart implements
		ISelectionListener, ISelectionChangedListener, IDoubleClickListener,
		ITabbedPropertySheetPageContributor, ICustomMenuContributor,
		ISaveablesSource, ISaveablePart {

	/** Stores the TreeViewer. */
	private TreeViewer viewer;

	/** Stores the menu manager. */
	private MenuManager menuManager;

	/** Stores the UI update job. */
	private final UIJob updateUI = new UIJob("Update Model Navigator") {
		@Override
		public IStatus runInUIThread(IProgressMonitor monitor) {
			if (viewer.getControl().isDisposed()) {
				return Status.OK_STATUS;
			}
			IActionService.INSTANCE.refresh();
			viewer.refresh();
			return Status.OK_STATUS;
		}
	};

	/** Constructor. */
	public NavigatorViewPart() {
		((NavigatorService) INavigatorService.INSTANCE)
				.setNavigatorViewPart(this);
	}

	/** {@inheritDoc} */
	@Override
	public void createPartControl(Composite parent) {
		viewer = new TreeViewer(parent, SWT.MULTI);

		// NOTE that the order of this important. See also JDT package
		// explorer.
		IDecoratorManager decoratorManager = PlatformUI.getWorkbench()
				.getDecoratorManager();
		viewer.setLabelProvider(new DecoratingLabelProvider(
				new NavigatorTreeLabelProvider(), decoratorManager
						.getLabelDecorator()));
		viewer.setContentProvider(new NavigatorTreeContentProvider());

		getSite().setSelectionProvider(viewer);
		createContextMenu();
		viewer.addSelectionChangedListener(this);

		viewer.setInput(IPersistencyService.INSTANCE);

		viewer.addDoubleClickListener(this);

		IActionService.INSTANCE.registerGlobalActions(getViewSite());

		PlatformUI.getWorkbench().getActiveWorkbenchWindow()
				.getSelectionService().addSelectionListener(this);

		// FIXME (FH): whats that good for?
		// if (viewer.getTree().getItems().length > 0) {
		// setActiveECPProject(viewer.getTree().getItem(0).getData());
		// viewer.getTree().select(viewer.getTree().getItem(0));
		// }
	}

	/** Creates the context menu. */
	private void createContextMenu() {
		menuManager = IContextMenuService.INSTANCE
				.createDefaultContextMenu(this);

		Menu contextMenu = menuManager.createContextMenu(viewer.getControl());
		viewer.getControl().setMenu(contextMenu);

		getSite().registerContextMenu(menuManager, viewer);
	}

	/** Creates the actions. */

	/** {@inheritDoc} */
	@Override
	public void setFocus() {
		viewer.getControl().setFocus();
	}

	/** {@inheritDoc} */
	@Override
	public void selectionChanged(IWorkbenchPart part, ISelection selection) {
		IActionService.INSTANCE.refresh();
		// TODO (FH): implement link with editor feature here
	}

	/** {@inheritDoc} */
	@Override
	public void selectionChanged(SelectionChangedEvent event) {
		IActionService.INSTANCE.refresh();
	}

	/** {@inheritDoc} */
	@Override
	public void dispose() {
		((NavigatorService) INavigatorService.INSTANCE)
				.setNavigatorViewPart(null);

		PlatformUI.getWorkbench().getActiveWorkbenchWindow()
				.getSelectionService().removeSelectionListener(this);
		getSite().setSelectionProvider(null);

		super.dispose();
	}

	/** {@inheritDoc} */
	@Override
	public void doubleClick(DoubleClickEvent event) {
		// delegate to the editor service
		if (event.getSelection() instanceof IStructuredSelection) {
			EObject element = EObjectSelectionUtils.getFirstElement(event
					.getSelection());
			if (element != null) {
				IEditorService.INSTANCE.openInEditor(element);
			}
		}
	}

	/** {@inheritDoc} */
	@Override
	public String getContributorId() {
		return IPropertiesService.TABBED_PROPERTY_CONTRIBUTOR_ID;
	}

	/**
	 * {@inheritDoc}
	 */
	@SuppressWarnings("rawtypes")
	@Override
	public Object getAdapter(Class adapter) {
		if (adapter == IPropertySheetPage.class) {
			return new TabbedPropertySheetPage(this);
		}
		return super.getAdapter(adapter);
	}

	/** {@inheritDoc} */
	@Override
	public List<IContributionItem> getContributedItems() {
		List<IContributionItem> result = new ArrayList<IContributionItem>();
		// FIXME (FH): provide a wrapper menu with the new menu
		// currently, only the entries are shown, but not a sub-menu
		result.add(new NewMenu());
		return result;
	}

	/** Refreshes the navigator view. */
	public void refresh() {
		if (INavigatorService.INSTANCE.getNavigatorView() != this) {
			return;
		}
		updateUI.schedule();
		firePropertyChange(IWorkbenchPartConstants.PROP_DIRTY);
	}

	/** {@inheritDoc} */
	@Override
	public Saveable[] getSaveables() {
		return INavigatorService.INSTANCE.getSaveables();
	}

	/** {@inheritDoc} */
	@Override
	public Saveable[] getActiveSaveables() {
		return INavigatorService.INSTANCE.getActiveSaveables();
	}

	/** {@inheritDoc} */
	@Override
	public void doSave(IProgressMonitor monitor) {
		// ignore. SaveableSource is used instead.
	}

	/** {@inheritDoc} */
	@Override
	public void doSaveAs() {
		// ignore
	}

	/** {@inheritDoc} */
	@Override
	public boolean isDirty() {
		return IPersistencyService.INSTANCE.isDirty();
	}

	/** {@inheritDoc} */
	@Override
	public boolean isSaveAsAllowed() {
		return false;
	}

	/** {@inheritDoc} */
	@Override
	public boolean isSaveOnCloseNeeded() {
		return isDirty();
	}
}
